# -*- coding: utf-8 -*-

import os
import logging
import matplotlib as mpl
from matplotlib.backends import qt_compat
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtWidgets import QAction
from matplotlib.backends.backend_qt5 import NavigationToolbar2QT
from PyQt5.QtCore import pyqtSignal, QSize

_translate = QtCore.QCoreApplication.translate

logger = logging.getLogger()
file_path = os.path.abspath(os.path.dirname(__file__))


class PamHyrNavigationToolbar2QT(NavigationToolbar2QT):
    """
    Cette classe est une personnalisation de la toolbar (NavigationToolbar2QT). Elle permet l'internationnalisation et
    l'ajout d'autres boutons à la toolbar initiale
    """
    isometric_signal = pyqtSignal(str)

    def __init__(self, canvas, parent):
        """
        Args:
            canvas: canvas de matplotlib
            parent: parent du canvas
        """
        self.my_canvas = canvas
        self.toolitems = [
            (None, None, None, None),

            ('Home', _translate("Toolbar", 'Vue originale'), 'home', 'home'),
            (None, None, None, None),

            ('Back', _translate("Toolbar", 'Retour à la vue précédente'), 'back', 'back'),
            ('Forward', _translate("Toolbar", 'Passer à la vue suivante'), 'forward', 'forward'),
            (None, None, None, None),

            ('Pan', _translate(
                "Toolbar",
                'Panoramique des axes avec la souris gauche, zoom avec la droite'
            ), 'move', 'pan'),
            (None, None, None, None),

            ('Zoom', _translate("Toolbar", 'Zoom'), 'zoom_to_rect', 'zoom'),
            (None, None, None, None),

            ('Isometric_view', _translate(
                "Toolbar", 'Vue isométrique (Shift+W)'
            ), '', 'isometric_view'),
            (None, None, None, None),

            ('GlobalView', _translate(
                "Toolbar", 'Vue globale automatique (Shift+X)'
            ), '', 'non_isometric_view'),

            (None, None, None, None),

            ('Save', _translate(
                "Toolbar", 'Enregistrer la figure'
            ), 'filesave', 'save_figure'),
            (None, None, None, None),
        ]

        NavigationToolbar2QT.__init__(self, canvas, parent)
        btn_size = QSize(30, 30)

        icon_zoom = QtGui.QIcon()
        icon_zoom.addPixmap(QtGui.QPixmap(
            f"{os.path.dirname(os.path.dirname(file_path))}/resources/icons/zoom.png"
        ))
        icon_btn_isometric_view = QtGui.QIcon()
        icon_btn_isometric_view.addPixmap(
            QtGui.QPixmap(f"{os.path.dirname(os.path.dirname(file_path))}/resources"
                          f"/icons/zoom_fit_11.png"))

        icon_btn_global_view = QtGui.QIcon()
        icon_btn_global_view.addPixmap(
            QtGui.QPixmap(f"{os.path.dirname(os.path.dirname(file_path))}/resources/icons"
                          f"/zoom_fit.png"))

        actions = self.findChildren(QAction)
        self._actions["zoom"].setIcon(icon_zoom)
        self._actions["isometric_view"].setIcon(icon_btn_isometric_view)
        self._actions["non_isometric_view"].setIcon(icon_btn_global_view)

        self.addSeparator()

    def save_figure(self, *args):
        filetypes = self.canvas.get_supported_filetypes_grouped()
        sorted_filetypes = sorted(filetypes.items())
        default_filetype = self.canvas.get_default_filetype()

        startpath = os.path.expanduser(mpl.rcParams['savefig.directory'])
        start = os.path.join(startpath, self.canvas.get_default_filename())
        filters = []
        selectedFilter = None
        for name, exts in sorted_filetypes:
            exts_list = " ".join(['*.%s' % ext for ext in exts])
            filter = '%s (%s)' % (name, exts_list)
            if default_filetype in exts:
                selectedFilter = filter
            filters.append(filter)
        filters = ';;'.join(filters)

        fname, filter = qt_compat._getSaveFileName(
            self.canvas.parent(),
            _translate("MainWindow_reach", "Choisissez un nom de fichier à sauvegarder"),
            start,
            filters, selectedFilter)

        if fname:
            if startpath != "":
                mpl.rcParams['savefig.directory'] = os.path.dirname(fname)

            try:
                self.canvas.figure.savefig(fname)
            except Exception as e:
                QtWidgets.QMessageBox.critical(
                    self, "Error saving file", str(e),
                    QtWidgets.QMessageBox.Ok, QtWidgets.QMessageBox.NoButton
                )

    def cursor1(self):
        logger.info("TODO: cursor1")

    def isometric_view(self):
        self.my_canvas.axes.axis("equal")
        self.my_canvas.figure.canvas.draw_idle()
        self.isometric_signal[str].emit("vue iso")

    def non_isometric_view(self):
        self.my_canvas.axes.axis("tight")
        self.my_canvas.toolbar.update()
        self.my_canvas.figure.canvas.draw_idle()

    def toolitems_translation(self):
        self._actions['home'].setToolTip(_translate("Toolbar", "Vue originale"))
        self._actions['back'].setToolTip(_translate("Toolbar", "Retour à la vue précédente"))
        self._actions['forward'].setToolTip(_translate("Toolbar", "Passer à la vue suivante"))
        self._actions['pan'].setToolTip(_translate("Toolbar", "Panoramique des axes avec la "
                                                              "souris gauche, zoom avec la droite"))
        self._actions['zoom'].setToolTip(_translate("Toolbar", "Zoom"))
        self._actions['save_figure'].setToolTip(_translate("Toolbar", "Enregistrer la figure"))
        self.action_isometric_view.setToolTip(_translate("Toolbar", "Vue isométrique (Shift+W)"))
        self.action_auto_global_view.setToolTip(_translate("Toolbar", "Vue globale automatique (Shift+X)"))
